var postLoadResources = new Array();

var amis =new Object();

amis.STATE_UNINITIALIZED=0;
amis.STATE_LOADING=1;
amis.STATE_LOADED_AND_PROCESSING=3;
amis.STATE_PROCESSED=4;
amis.STATE_LOADED_AND_WAITING=5;

    function el(id) {
      return document.getElementById(id);
    }


/*--- content loader object for cross-browser requests ---*/
amis.PostLoadResource=function           
                   ( elementId
                   , url
                   , requireProxy // boolean indicating whether or not the resource must be acquired through a proxy from a remote domain
                   , label
                   , processor // function reference of function that will process the resource when received
                   , dependsOn // other PostLoadResources this PLR may depend on, such as an XSLT that an XML depends on for being processed
                   , refreshTime // time interval in seconds after which the resource should be reloaded and reprocessed
                   ) {
  this.elementId=elementId;
  this.url=url;
  this.requireProxy = requireProxy;
  this.state = amis.STATE_UNINITIALIZED;
  this.label = label;
  this.req = null;
  this.id = null;
  this.processor= processor;
  this.refreshTime = refreshTime;
  this.dependsOn = dependsOn;
  this.dependents = new Array();
  // also tell every PLR that is depended on to notify this PLR when they are processed
  if (dependsOn) {
    var i=0;
    for (i=0;i<dependsOn.length;i++) {
      dependsOn[i].dependents.push(this);
    }//for
  } // if dependsOn
}

// this function is to be called for a PLR that depends on another PLR when
// that other PLR has been loaded so it (the listener or observer) can evaluate 
// whether all its dependencies are available and so it can start processing itself
amis.PostLoadResource.prototype.notify=function(plr){
  if (this.state == amis.STATE_LOADED_AND_WAITING) {
    startProcessing(this);
  }
}

  // create a new PLR object and add it to the array of PLR objects to be dealt with when the page has loaded
  function addPostLoadResource
           ( elementId
           , url
           , requireProxy // boolean indicating whether or not the resource must be acquired through a proxy from a remote domain
           , label
           , processor // function reference of function that will process the resource when received
           , dependsOn
           , refreshTime // in seconds
           ) {
     var plr = new amis.PostLoadResource(elementId, url, requireProxy,label, processor, dependsOn, refreshTime);
     var size = postLoadResources.push( plr ); // add a new PostLoadResource object to the array
     plr.id = size -1; // ensure the plr objects knows where it sits in the postLoadResources array
     return plr;
  }


  function postLoadSingleResource(postLoadResource) {
      postLoadResource.state = amis.STATE_LOADING;
      if (postLoadResource.requireProxy) {
        loader = new net.ContentLoader('proxy?remotehost='+postLoadResource.url, processPostLoad);
      }
      else {
        loader = new net.ContentLoader(postLoadResource.url, processPostLoad);
      }
      loader.parameter = postLoadResource;
  }// goPostLoadResources


  function goPostLoadResources() {
    // iterate array postLoadResources
    var i=0;
    var loader;
    for (i=0;i<postLoadResources.length;i++) {
      if (postLoadResources[i].state == amis.STATE_UNINITIALIZED) {
        postLoadSingleResource(postLoadResources[i]);
      }
    }// for
  }// goPostLoadResources

  // this function checks whether the PLR has dependencies on other PLRs
  // that are not yet loaded
  function dependsOnUnloaded( postLoadResource) {
    // check if the postLoadResource depends on other plrs that are not yet loaded
    // if so, it goes into the waiting room - state= amis.STATE_LOADED_AND_WAITING
    if (postLoadResource.dependsOn) {
      var i=0;
      for (i=0;i<postLoadResource.dependsOn.length;i++) {
        if (postLoadResource.dependsOn[i].state <  amis.STATE_LOADED_AND_PROCESSING) {
          return true;
        }
      }//for
    } // if dependsOn
    return false;
  }//dependsOnUnloaded


  // This function is called by the ContentLoader when the AjaxRequest is successfully completed
  // and the requested content is available for further processing.
  function processPostLoad() {
    var postLoadResource = this.parameter;
    postLoadResource.req = this.req;
    postLoadResource.state = amis.STATE_LOADED_AND_PROCESSING;
    // now tell all dependents that this plr is being processed
    if (postLoadResource.dependents) {
      var i=0;
      for (i=0;i<postLoadResource.dependents.length;i++) {
        postLoadResource.dependents[i].notify(postLoadResource);
      }//for
    } // if dependents
    startProcessing( postLoadResource);
  }// processPostLoad

  function handleTimeOut(id) {
    postLoadSingleResource(postLoadResources[id]);
  }

  // This function is called when a postLoadResources is loaded
  // and all its dependents have been loaded as well
  // and the requested content is available for further processing.
  // There is a number of things we can do in terms of post-processing
  // * use the contents as is
  // * xslt transform the contents
  // * HTML scrape the contents
  // * split CSV or otherwise pre-process the results
  // taking the result of this first step, we can:
  // - paste into the innerHTML of a specified element, for example a DIV or TD
  // - copy into the value attribute of a Form element like a TEXTAREA or INPUT
  // - process into <OPTION> elements under a select
  // - copy into JavaScript memorystructure
  // - call custom processor to anything special like populate SVG object
  function startProcessing(postLoadResource) {
    // check if the postLoadResource depends on other plrs that are not yet processed
    // if so, it goes into the waiting room - state= amis.STATE_LOADED_AND_WAITING
    if (dependsOnUnloaded( postLoadResource)) { 
       postLoadResource.state = amis.STATE_LOADED_AND_WAITING;
       return;
    }
    if (postLoadResource.processor) 
    { 
      postLoadResource.processor.call(postLoadResource);
      }
    else {
      if (postLoadResource.elementId) {
        // go find element and load contents
        var element = el(postLoadResource.elementId);
        if (element.value) {
          element.value = postLoadResource.req.responseText;
        }
        else {
          element.innerHTML = postLoadResource.req.responseText;
        }
      }
    }// processor
    postLoadResource.state = amis.STATE_PROCESSED;
    if (postLoadResource.refreshTime) {
      setTimeout("handleTimeOut('"+postLoadResource.id+"')", postLoadResource.refreshTime*1000);
    }
  }// startProcessing
  

